@@ -497,7 +497,7 @@ QVariant KateFileTreeModel::data( const QModelIndex &index, int role ) const
     case Qt::ToolTipRole: {
       QString tooltip = item->path();
       if (item->flag(ProxyItem::DeletedExternally) || item->flag(ProxyItem::ModifiedExternally)) {
-        tooltip = i18nc("%1 is the full path", "<p><b>%1</b></p><p>The document has been modified by another application.</p>").arg(item->path());
+        tooltip = i18nc("%1 is the full path", "<p><b>%1</b></p><p>The document has been modified by another application.</p>", item->path());
       }
 
       return tooltip;
