@@ -24,6 +24,7 @@
 #include <QTextEdit>
 #include <QMultiMap>
 #include <QTimer>
+#include <QVarLengthArray>
 
 #include <klocale.h>
 #include <kiconloader.h>
@@ -951,7 +952,9 @@ void KateCompletionModel::setCurrentCompletion( KTextEditor::CodeCompletionModel
         needsReset |= changeCompletions(g, changeType);
     }
   }
-  updateBestMatches();
+
+  // NOTE: best matches are also updated in resort
+  resort();
 
   kDebug()<<"needsReset"<<needsReset;
   if(needsReset)
@@ -1456,15 +1459,29 @@ bool KateCompletionModel::Item::operator <( const Item & rhs ) const
 
   //kDebug( 13035 ) << c1 << " c/w " << c2 << " -> " << (model->isSortingReverse() ? ret > 0 : ret < 0) << " (" << ret << ")";
 
+  const bool isBad = m_sourceRow.second.data(CodeCompletionModel::UnimportantItemRole).toBool();
+  const bool otherIsBad = rhs.m_sourceRow.second.data(CodeCompletionModel::UnimportantItemRole).toBool();
+  if( isBad && !otherIsBad ) {
+    return false;
+  }
+  if( otherIsBad && !isBad ) {
+    return true;
+  }
+
+  if( matchCompletion < rhs.matchCompletion ) {
+    // enums are ordered in the order items should be displayed
+    return true;
+  }
+  if( matchCompletion > rhs.matchCompletion ) {
+    return false;
+  }
+
   if( model->isSortingByInheritanceDepth() )
     ret = inheritanceDepth - rhs.inheritanceDepth;
 
   if (ret == 0 && model->isSortingAlphabetical()) {
-    if(!m_completionSortingName.isEmpty() && !rhs.m_completionSortingName.isEmpty())
-      //Shortcut, plays a role in this tight loop
-      ret = QString::compare(m_completionSortingName, rhs.m_completionSortingName);
-    else
-      ret = QString::compare(completionSortingName(), rhs.completionSortingName()); //Do not use localeAwareCompare, because it is simply too slow for a list of about 1000 items
+    // Do not use localeAwareCompare, because it is simply too slow for a list of about 1000 items
+    ret = QString::compare(m_nameColumn, rhs.m_nameColumn, model->sortingCaseSensitivity());
   }
 
   if( ret == 0 ) {
@@ -1475,17 +1492,6 @@ bool KateCompletionModel::Item::operator <( const Item & rhs ) const
   return ret < 0;
 }
 
-QString KateCompletionModel::Item::completionSortingName( ) const
-{
-  if(m_completionSortingName.isEmpty()) {
-    m_completionSortingName = m_nameColumn;
-    if (model->sortingCaseSensitivity() == Qt::CaseInsensitive)
-      m_completionSortingName = m_completionSortingName.toLower();
-  }
-
-  return m_completionSortingName;
-}
-
 void KateCompletionModel::Group::addItem( Item i, bool notifyModel )
 {
   if (isEmpty)
@@ -1557,15 +1563,8 @@ void KateCompletionModel::setSortingAlphabetical( bool alphabetical )
 
 void KateCompletionModel::Group::resort( )
 {
-  qStableSort(prefilter.begin(), prefilter.end());
-  //int oldRowCount = filtered.count();
-  filtered.clear();
-  foreach (const Item& i, prefilter)
-    if (i.isVisible())
-      filtered.append(i);
-
+  qStableSort(filtered.begin(), filtered.end());
   model->hideOrShowGroup(this);
-  //Q_ASSERT(filtered.count() == oldRowCount);
 }
 
 void KateCompletionModel::setSortingCaseSensitivity( Qt::CaseSensitivity cs )
@@ -1584,6 +1583,8 @@ void KateCompletionModel::resort( )
   foreach (Group* g, m_emptyGroups)
     g->resort();
 
+  // call updateBestMatches here, so they are moved to the top again.
+  updateBestMatches();
   emit contentGeometryChanged();
 }
 
@@ -1751,6 +1752,106 @@ bool KateCompletionModel::shouldMatchHideCompletionList() const {
   return doHide;
 }
 
+static inline bool matchesAbbreviationHelper(const QString& word, const QString& typed, const QVarLengthArray<int, 32>& offsets,
+                                             int& depth, int atWord = -1, int i = 0) {
+  int atLetter = 1;
+  for ( ; i < typed.size(); i++ ) {
+    const QChar c = typed.at(i).toLower();
+    bool haveNextWord = offsets.size() > atWord + 1;
+    bool canCompare = atWord != -1 && word.size() > offsets.at(atWord) + atLetter;
+    if ( canCompare && c == word.at(offsets.at(atWord) + atLetter).toLower() ) {
+      // the typed letter matches a letter after the current word beginning
+      if ( ! haveNextWord || c != word.at(offsets.at(atWord + 1)).toLower() ) {
+        // good, simple case, no conflict
+        atLetter += 1;
+        continue;
+      }
+      // For maliciously crafted data, the code used here theoretically can have very high
+      // complexity. Thus ensure we don't run into this case, by limiting the amount of branches
+      // we walk through to 128.
+      depth++;
+      if ( depth > 128 ) {
+        return false;
+      }
+      // the letter matches both the next word beginning and the next character in the word
+      if ( haveNextWord && matchesAbbreviationHelper(word, typed, offsets, depth, atWord + 1, i + 1) ) {
+        // resolving the conflict by taking the next word's first character worked, fine
+        return true;
+      }
+      // otherwise, continue by taking the next letter in the current word.
+      atLetter += 1;
+      continue;
+    }
+    else if ( haveNextWord && c == word.at(offsets.at(atWord + 1)).toLower() ) {
+      // the typed letter matches the next word beginning
+      atWord++;
+      atLetter = 1;
+      continue;
+    }
+    // no match
+    return false;
+  }
+  // all characters of the typed word were matched
+  return true;
+}
+
+bool KateCompletionModel::matchesAbbreviation(const QString& word, const QString& typed)
+{
+  // A mismatch is very likely for random even for the first letter,
+  // thus this optimization makes sense.
+  if ( word.at(0).toLower() != typed.at(0).toLower() ) {
+    return false;
+  }
+
+  // First, check if all letters are contained in the word in the right order.
+  int atLetter = 0;
+  foreach ( const QChar c, typed ) {
+    while ( c.toLower() != word.at(atLetter).toLower() ) {
+      atLetter += 1;
+      if ( atLetter >= word.size() ) {
+        return false;
+      }
+    }
+  }
+
+  bool haveUnderscore = true;
+  QVarLengthArray<int, 32> offsets;
+  // We want to make "KComplM" match "KateCompletionModel"; this means we need
+  // to allow parts of the typed text to be not part of the actual abbreviation,
+  // which consists only of the uppercased / underscored letters (so "KCM" in this case).
+  // However it might be ambigous whether a letter is part of such a word or part of
+  // the following abbreviation, so we need to find all possible word offsets first,
+  // then compare.
+  for ( int i = 0; i < word.size(); i++ ) {
+    const QChar c = word.at(i);
+    if ( c == QLatin1Char('_') ) {
+      haveUnderscore = true;
+    } else if ( haveUnderscore || c.isUpper() ) {
+      offsets.append(i);
+      haveUnderscore = false;
+    }
+  }
+  int depth = 0;
+  return matchesAbbreviationHelper(word, typed, offsets, depth);
+}
+
+static inline bool containsAtWordBeginning(const QString& word, const QString& typed, Qt::CaseSensitivity caseSensitive) {
+  for(int i = 1; i < word.size(); i++) {
+      // The current position is a word beginning if the previous character was an underscore
+      // or if the current character is uppercase. Subsequent uppercase characters do not count,
+      // to handle the special case of UPPER_CASE_VARS properly.
+      const QChar c = word.at(i);
+      const QChar prev = word.at(i-1);
+      if(!(prev == QLatin1Char('_') || (c.isUpper() && !prev.isUpper()))) {
+        continue;
+      }
+      if(word.midRef(i).startsWith(typed, caseSensitive)) {
+        return true;
+      }
+    }
+    return false;
+};
+
 KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
 {
   // Check to see if the item is matched by the current completion string
@@ -1763,8 +1864,27 @@ KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
    // Hehe, everything matches nothing! (ie. everything matches a blank string)
    if (match.isEmpty())
      return PerfectMatch;
-  
+   if (m_nameColumn.isEmpty())
+     return NoMatch;
+
   matchCompletion = (m_nameColumn.startsWith(match, model->matchCaseSensitivity()) ? StartsWithMatch : NoMatch);
+  if(matchCompletion == NoMatch) {
+    // if no match, try for "contains"
+    // Only match when the occurence is at a "word" beginning, marked by
+    // an underscore or a capital. So Foo matches BarFoo and Bar_Foo, but not barfoo.
+    // Starting at 1 saves looking at the beginning of the word, that was already checked above.
+    if(containsAtWordBeginning(m_nameColumn, match, model->matchCaseSensitivity())) {
+      matchCompletion = ContainsMatch;
+    }
+  }
+
+  if(matchCompletion == NoMatch && !m_nameColumn.isEmpty() && !match.isEmpty())
+  {
+    // if still no match, try abbreviation matching
+    if(matchesAbbreviation(m_nameColumn, match)) {
+      matchCompletion = AbbreviationMatch;
+    }
+  }
 
   if(matchCompletion && match.length() == m_nameColumn.length()) {
     matchCompletion = PerfectMatch;
