@@ -110,12 +110,15 @@ class KATEPART_TESTS_EXPORT KateDocument : public KTextEditor::Document,
     
     using ReadWritePart::closeUrl;
     virtual bool closeUrl();
+    
+    virtual bool openUrl( const KUrl &url );
 
     virtual KTextEditor::Editor *editor ();
 
     KTextEditor::Range rangeOnLine(KTextEditor::Range range, int line) const;
 
-    
+  private:
+    void showAndSetOpeningErrorAccess();
   /*
    * Overload this to have on-demand view creation
    */
@@ -711,13 +714,16 @@ Q_SIGNALS:
 
     bool ownedView(KateView *);
 
-    uint toVirtualColumn( const KTextEditor::Cursor& );
+    int toVirtualColumn( int line, int column ) const;
+    int toVirtualColumn( const KTextEditor::Cursor& ) const;
+    int fromVirtualColumn( int line, int column ) const;
+    int fromVirtualColumn( const KTextEditor::Cursor& ) const;
+
     void newLine( KateView*view ); // Changes input
     void backspace(     KateView *view, const KTextEditor::Cursor& );
     void del(           KateView *view, const KTextEditor::Cursor& );
     void transpose(     const KTextEditor::Cursor& );
-
-    void paste ( KateView* view, const QString &s );
+    void paste ( KateView* view, const QString &text );
 
   public:
     void indent ( KTextEditor::Range range, int change );
@@ -811,6 +817,7 @@ Q_SIGNALS:
     bool removeStartLineCommentFromSelection( KateView *view, int attrib=0 );
 
   public:
+    // KDE5: rename to wordAt(), add wordRangeAt(), see ktexteditor/document.h
     QString getWord( const KTextEditor::Cursor& cursor );
 
   public:
@@ -1217,6 +1224,10 @@ Q_SIGNALS:
      */
     bool m_readWriteStateBeforeLoading;
     
+    /**
+     * if the document is untitled
+     */
+    bool m_isUntitled;
     /**
      * loading job, we want to cancel with cancel in the loading message
      */
