@@ -2,6 +2,7 @@
  *
  *  Copyright (C) 2008-2011 Erlend Hamberg <ehamberg@gmail.com>
  *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+ *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -42,6 +43,11 @@ KateViInsertMode::KateViInsertMode( KateViInputModeManager *viInputModeManager,
   m_eolPos = 0;
   m_count = 1;
   m_countedRepeatsBeginOnNewLine = false;
+
+  m_isExecutingCompletion = false;
+
+  connect(doc(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)),
+          this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
 }
 
 KateViInsertMode::~KateViInsertMode()
@@ -186,6 +192,11 @@ bool KateViInsertMode::commandMoveOneWordRight()
   Cursor c( m_view->cursorPosition() );
   c = findNextWordStart( c.line(), c.column() );
 
+  if (!c.isValid())
+  {
+    c = doc()->documentEnd();
+  }
+
   updateCursor( c );
   return true;
 }
@@ -320,6 +331,22 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
     case Qt::Key_PageDown:
       m_view->pageDown();
       return true;
+    case Qt::Key_Enter:
+    case Qt::Key_Return:
+      if (m_view->completionWidget()->isCompletionActive() && !m_viInputModeManager->isReplayingMacro() && !m_viInputModeManager->isReplayingLastChange())
+      {
+        // Filter out Enter/ Return's that trigger a completion when recording macros/ last change stuff; they
+        // will be replaced with the special code "ctrl-space".
+        // (This is why there is a "!m_viInputModeManager->isReplayingMacro()" above.)
+        m_viInputModeManager->doNotLogCurrentKeypress();
+
+        m_isExecutingCompletion = true;
+        m_textInsertedByCompletion.clear();
+        m_view->completionWidget()->execute();
+        completionFinished();
+        m_isExecutingCompletion = false;
+        return true;
+      }
     default:
       return false;
       break;
@@ -332,7 +359,18 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
       return true;
       break;
     case Qt::Key_Space:
-      commandCompleteNext();
+      // We use Ctrl-space as a special code in macros/ last change, which means: if replaying
+      // a macro/ last change, fetch and execute the next completion for this macro/ last change ...
+      if (!m_viInputModeManager->isReplayingMacro() && !m_viInputModeManager->isReplayingLastChange())
+      {
+        commandCompleteNext();
+        // ... therefore, we should not record ctrl-space indiscriminately.
+        m_viInputModeManager->doNotLogCurrentKeypress();
+      }
+      else
+      {
+        replayCompletion();
+      }
       return true;
       break;
     case Qt::Key_C:
@@ -348,11 +386,17 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
       return true;
       break;
     case Qt::Key_N:
-      commandCompleteNext();
+      if (!m_viInputModeManager->isReplayingMacro())
+      {
+        commandCompleteNext();
+      }
       return true;
       break;
     case Qt::Key_P:
-      commandCompletePrevious();
+      if (!m_viInputModeManager->isReplayingMacro())
+      {
+        commandCompletePrevious();
+      }
       return true;
       break;
     case Qt::Key_T:
@@ -410,11 +454,7 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
 
     // Was waiting for register for Ctrl-R
     if (m_keys == "cR"){
-        QChar key = KateViKeyParser::self()->KeyEventToQChar(
-                    e->key(),
-                    e->text(),
-                    e->modifiers(),
-                    e->nativeScanCode() );
+        QChar key = KateViKeyParser::self()->KeyEventToQChar(*e);
         key = key.toLower();
 
         // is it register ?
@@ -438,6 +478,7 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
 // ctrl-c is used to exit insert mode this is not done.
 void KateViInsertMode::leaveInsertMode( bool force )
 {
+    m_view->abortCompletion();
     if ( !force )
     {
       if ( m_blockInsert != None ) { // block append/prepend
@@ -500,9 +541,9 @@ void KateViInsertMode::leaveInsertMode( bool force )
                   doc()->insertText( m_view->cursorPosition(), added );
               }
           }
-          m_countedRepeatsBeginOnNewLine = false;
       }
     }
+    m_countedRepeatsBeginOnNewLine = false;
     startNormalMode();
 }
 
@@ -530,3 +571,127 @@ void KateViInsertMode::setBlockAppendMode( KateViRange blockRange, BlockInsert b
         kDebug( 13070 ) << "cursor moved. ignoring block append/prepend";
     }
 }
+
+void KateViInsertMode::completionFinished()
+{
+  KateViInputModeManager::Completion::CompletionType completionType = KateViInputModeManager::Completion::PlainText;
+  if (m_view->cursorPosition() != m_textInsertedByCompletionEndPos)
+  {
+    completionType = KateViInputModeManager::Completion::FunctionWithArgs;
+  }
+  else if (m_textInsertedByCompletion.endsWith("()") || m_textInsertedByCompletion.endsWith("();"))
+  {
+    completionType = KateViInputModeManager::Completion::FunctionWithoutArgs;
+  }
+  m_viInputModeManager->logCompletionEvent(KateViInputModeManager::Completion(m_textInsertedByCompletion, KateViewConfig::global()->wordCompletionRemoveTail(), completionType));
+}
+
+void KateViInsertMode::replayCompletion()
+{
+  const KateViInputModeManager::Completion completion = m_viInputModeManager->nextLoggedCompletion();
+  // Find beginning of the word.
+  Cursor cursorPos = m_view->cursorPosition();
+  Cursor wordStart = Cursor::invalid();
+  if (!doc()->character(cursorPos).isLetterOrNumber() && doc()->character(cursorPos) != '_')
+  {
+    cursorPos.setColumn(cursorPos.column() - 1);
+  }
+  while (cursorPos.column() >= 0 && (doc()->character(cursorPos).isLetterOrNumber() || doc()->character(cursorPos) == '_'))
+  {
+    wordStart = cursorPos;
+    cursorPos.setColumn(cursorPos.column() - 1);
+  }
+  // Find end of current word.
+  cursorPos = m_view->cursorPosition();
+  Cursor wordEnd = Cursor(cursorPos.line(), cursorPos.column() - 1);
+  while (cursorPos.column() < doc()->lineLength(cursorPos.line()) && (doc()->character(cursorPos).isLetterOrNumber() || doc()->character(cursorPos) == '_'))
+  {
+    wordEnd = cursorPos;
+    cursorPos.setColumn(cursorPos.column() + 1);
+  }
+  QString completionText = completion.completedText();
+  const Range currentWord = Range(wordStart, Cursor(wordEnd.line(), wordEnd.column() + 1));
+  // Should we merge opening brackets? Yes, if completion is a function with arguments and after the cursor
+  // there is (optional whitespace) followed by an open bracket.
+  int offsetFinalCursorPosBy = 0;
+  if (completion.completionType() == KateViInputModeManager::Completion::FunctionWithArgs)
+  {
+    const int nextMergableBracketAfterCursorPos = findNextMergeableBracketPos(currentWord.end());
+    if (nextMergableBracketAfterCursorPos != -1)
+    {
+      if (completionText.endsWith("()"))
+      {
+        // Strip "()".
+        completionText = completionText.left(completionText.length() - 2);
+      }
+      else if (completionText.endsWith("();"))
+      {
+        // Strip "();".
+        completionText = completionText.left(completionText.length() - 3);
+      }
+      // Ensure cursor ends up after the merged open bracket.
+      offsetFinalCursorPosBy = nextMergableBracketAfterCursorPos + 1;
+    }
+    else
+    {
+      if (!completionText.endsWith("()") && !completionText.endsWith("();"))
+      {
+        // Original completion merged with an opening bracket; we'll have to add our own brackets.
+        completionText.append("()");
+      }
+      // Position cursor correctly i.e. we'll have added "functionname()" or "functionname();"; need to step back by
+      // one or two to be after the opening bracket.
+      offsetFinalCursorPosBy = completionText.endsWith(";") ? -2 : -1;
+    }
+  }
+  Cursor deleteEnd =  completion.removeTail() ? currentWord.end() :
+                                                Cursor(m_view->cursorPosition().line(), m_view->cursorPosition().column() + 0);
+
+  if (currentWord.isValid())
+  {
+    doc()->removeText(Range(currentWord.start(), deleteEnd));
+    doc()->insertText(currentWord.start(), completionText);
+  }
+  else
+  {
+    doc()->insertText(m_view->cursorPosition(), completionText);
+  }
+
+  if (offsetFinalCursorPosBy != 0)
+  {
+    m_view->setCursorPosition(Cursor(m_view->cursorPosition().line(), m_view->cursorPosition().column() + offsetFinalCursorPosBy));
+  }
+
+  if (!m_viInputModeManager->isReplayingLastChange())
+  {
+    Q_ASSERT(m_viInputModeManager->isReplayingMacro());
+    // Post the completion back: it needs to be added to the "last change" list ...
+    m_viInputModeManager->logCompletionEvent(completion);
+    // ... but don't log the ctrl-space that led to this call to replayCompletion(), as
+    // a synthetic ctrl-space was just added to the last change keypresses by logCompletionEvent(), and we don't
+    // want to duplicate them!
+    m_viInputModeManager->doNotLogCurrentKeypress();
+  }
+}
+
+
+int KateViInsertMode::findNextMergeableBracketPos(const Cursor& startPos)
+{
+  const QString lineAfterCursor = doc()->text(Range(startPos, Cursor(startPos.line(), doc()->lineLength(startPos.line()))));
+  QRegExp whitespaceThenOpeningBracket("^\\s*(\\()");
+  int nextMergableBracketAfterCursorPos = -1;
+  if (lineAfterCursor.contains(whitespaceThenOpeningBracket))
+  {
+    nextMergableBracketAfterCursorPos =  whitespaceThenOpeningBracket.pos(1);
+  }
+  return nextMergableBracketAfterCursorPos;
+}
+
+void KateViInsertMode::textInserted(KTextEditor::Document* document, KTextEditor::Range range)
+{
+  if (m_isExecutingCompletion)
+  {
+    m_textInsertedByCompletion += document->text(range);
+    m_textInsertedByCompletionEndPos = range.end();
+  }
+}
\ No newline at end of file
